/* honeytrap.c
 * Copyright (C) 2005-2006 Tillmann Werner <tillmann.werner@gmx.de>
 *
 * This file is free software; as a special exception the author gives
 * unlimited permission to copy and/or distribute it, with or without
 * modifications, as long as this notice is preserved.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

#include <stdlib.h>
#include <netdb.h>
#include <string.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>

#include "honeytrap.h"
#include "logging.h"
#include "plugin.h"
#include "ctrl.h"
#include "response.h"
#include "sniffer.h"
#include "readconf.h"


int main(int argc, char **argv) {
	int i;

	/* initial configuration, will be overridden by config file and command line options  */

	/* save command line arguments */
	arg_c = argc;
	arg_v = argv;

	promisc_mode	= 0;		/* no promisc mode */
	mirror_mode	= 0;		/* mirror mode is not on by default */
	daemonize	= 1;		/* default is to daemonize */
	p2p_offset	= 0;		/* will be set if we're sniffing on a p2p interface */

	log_level	= LOG_NOTICE;	/* default log level */
	logfile_fd	= STDOUT_FILENO;/* default logfile, stdout will be replaced by logfile_fd */
	
	u_id=0;				/* root privileges per default */
	g_id=0;

	conn_timeout	= 120;		/* 2 minutes connect timeout */
	read_timeout	= 1;		/* 1 second read timeout */
	m_read_timeout	= 60;		/* 1 minute read timeout for mirror connections */
	
	conffile_name = malloc(30);	/* space for '/etc/honeytrap/honeytrap.conf' */
	strcpy(conffile_name, "/etc/honeytrap/honeytrap.conf");

	pidfile_name	= malloc(23);	/* space for '/var/run/honeytrap.pid' */
	strcpy(pidfile_name, "/var/run/honeytrap.pid");
	
	logfile_name = malloc(23);	/* space for '/var/log/honeytrap.log' */
	strcpy(logfile_name, "/var/log/honeytrap.log");

	logfile_name = malloc(34);	/* space for '/var/spool/honeytrap/honeytrap.db' */
	strcpy(logfile_name, "/var/spool/honeytrap/honeytrap.db");

	response_dir = malloc(25);	/* space for '/etc/honeytrap/responses' */
	strcpy(response_dir, "/etc/honeytrap/responses");

	plugin_dir = malloc(23);	/* space for '/etc/honeytrap/plugins' */
	strcpy(plugin_dir, "/etc/honeytrap/plugins");

	attacks_dir = malloc(21);	/* space for '/var/spool/honeytrap' */
	strcpy(attacks_dir, "/var/spool/honeytrap");

	dlsave_dir = malloc(21);	/* space for '/var/spool/honeytrap' */
	strcpy(dlsave_dir, "/var/spool/honeytrap");


	/* configure honeytrap */
	configure(arg_c, arg_v);

	
	/* daemonize (detach from console) */
	if (daemonize) do_daemonize();


	/* create pid file */
	create_pid_file();

	
	/* sniff RST packets */
	if (start_sniffer() < 0) clean_exit(0);
	
	return(0);
}
