/* interface.c
 * Copyright (C) 2005 Tillmann Werner <tillmann.werner@gmx.de>
 *
 * This file is free software; as a special exception the author gives
 * unlimited permission to copy and/or distribute it, with or without
 * modifications, as long as this notice is preserved.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * This code is based on examples in R. Stevens' Unix Network Programming.
 */

#include <string.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <stdio.h>

#include "interface.h"

struct if_data *get_if_data(int family, const char *ifname) {
	struct if_data	*interface, *ifhead;
	int		sockfd, len, lastlen, flags;
	char		*ptr, *buf, *cptr;
	struct ifconf	ifc;
	struct ifreq	*ifr, ifrcopy;
	struct sockaddr_in *sinptr;
	struct if_data *d_interface;
	struct sockaddr_in *d_ip_addr;

	sockfd = socket(AF_INET, SOCK_DGRAM, 0);
	lastlen = 0;
	len = 100 * sizeof(struct ifreq);
	interface = NULL;

	for(;;) {
		buf = (char *) malloc(len);
		ifc.ifc_len = len;
		ifc.ifc_buf = buf;
		if (ioctl(sockfd, SIOCGIFCONF, &ifc) < 0) {
			perror("ioctl error");
			exit(1);
		} else {
			if (ifc.ifc_len == lastlen) break;
			lastlen = ifc.ifc_len;
		}
		len += 10 * sizeof(struct ifreq);
		free(buf);
	}

	for (ptr = buf; ptr < buf + ifc.ifc_len;) {
		ifr = (struct ifreq *) ptr;

#ifdef HAVE_SOCKADDR_SA_LEN
		len = max(sizeof(struct sockaddr), ifr->ifr_addr.sa_len);
#else
		switch (ifr->ifr_addr.sa_family) {
		case AF_INET:	
		default:	
			len = sizeof(struct sockaddr);
			break;
		}
#endif	/* HAVE_SOCKADDR_SA_LEN */

		ptr += sizeof(ifr->ifr_name) + len;			/* for next one in buffer */

		if (ifr->ifr_addr.sa_family != family)  continue;	/* ignore device if family is not IPv4 */

		if (strcmp(ifr->ifr_name, ifname) == 0) {
			if ( (cptr = strchr(ifr->ifr_name, ':')) != NULL) *cptr = 0;	/* replace colon will null */

			ifrcopy = *ifr;
			ioctl(sockfd, SIOCGIFFLAGS, &ifrcopy);
			flags = ifrcopy.ifr_flags;
			if ((flags & IFF_UP) == 0) return(NULL);	/* return if interface not up */

			interface = (struct if_data *) calloc(1, sizeof(struct if_data));

			interface->if_flags = flags;	/* IFF_xxx values */
			memcpy(interface->if_name, ifr->ifr_name, IF_NAME);
			interface->if_name[IF_NAME-1] = '\0';

			switch (ifr->ifr_addr.sa_family) {
			case AF_INET:
				sinptr = (struct sockaddr_in *) &ifr->ifr_addr;
				if (interface->if_addr == NULL) {
					interface->if_addr = (struct sockaddr *) calloc(1, sizeof(struct sockaddr_in));
					memcpy(interface->if_addr, sinptr, sizeof(struct sockaddr_in));
				}
				break;
			default:
				break;
			}
		} else continue;
	}
	free(buf);
	return(interface);
}


void free_if_data(struct if_data *interface) {
	if (interface) {
		if (interface->if_addr != NULL) free(interface->if_addr);
		free(interface);
	}
}
