/* signals.c
 * Copyright (C) 2005-2006 Tillmann Werner <tillmann.werner@gmx.de>
 *
 * This file is free software; as a special exception the author gives
 * unlimited permission to copy and/or distribute it, with or without
 * modifications, as long as this notice is preserved.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

#include <sys/types.h>
#include <unistd.h>
#include <signal.h>
#include <sys/wait.h>
#include <stdio.h>

#include "logging.h"
#include "honeytrap.h"
#include "ctrl.h"
#include "readconf.h"

#define MASTER_PROCESS (pid = getpid()) == parent_pid

void handle_signal(int sig) {
	char log_zeit[20];
	int i, stat;
	pid_t pid, ppid;

	switch(sig) {
		case SIGHUP:
			logmsg(LOG_DEBUG, 1, "SIGHUP received. Reconfiguring honeytrap.\n");
			logmsg(LOG_NOTICE, 1, "---- Reloading configuration ----\n");
			pcap_close(tcp_sniffer);

			/* unloading plugins */
			unload_plugins();
			
			/* configure honeytrap */
			configure(arg_c, arg_v);

			/* sniff RST packets */
			if (start_sniffer() < 0) clean_exit(0);
		case SIGINT:
			logmsg(LOG_DEBUG, 1, "SIGINT received.\n");
			if (MASTER_PROCESS) {
				if (kill(0-getpgrp(), SIGINT) == 0) {
					logmsg(LOG_DEBUG, 1, "SIGINT was successfully sent to process group.\n");
					/* wait for children */
					sleep(2);
				} else logmsg(LOG_ERR, 1, "Error sending SIGINT to process group.\n");
				clean_exit(0);
			} else _exit(0);
		case SIGQUIT:
			logmsg(LOG_DEBUG, 1, "SIGQUIT received.\n");
			if (MASTER_PROCESS) {
				if (kill(0-getpgrp(), SIGQUIT) == 0)
					logmsg(LOG_DEBUG, 1, "SIGQUIT was successfully sent to process group.\n");
				else logmsg(LOG_ERR, 1, "Error sending SIGQUIT to process group.\n");
				clean_exit(0);
			} else _exit(0);
		case SIGTERM:
			logmsg(LOG_DEBUG, 1, "SIGTERM received.\n");
			if (MASTER_PROCESS) {
				if (kill(0-getpgrp(), SIGTERM) == 0) {
					logmsg(LOG_DEBUG, 1, "SIGTERM was successfully sent to process group.\n");
					/* wait for children */
					sleep(2);
				} else logmsg(LOG_ERR, 1, "Error sending SIGTERM to process group.\n");
				clean_exit(0);
			} else _exit(0);
		case SIGCHLD:
			logmsg(LOG_DEBUG, 1, "SIGCHILD received.\n");
			while ((pid = waitpid(-1, NULL, WNOHANG)) > 0) {
				logmsg(LOG_DEBUG, 1, "Process %d terminated.\n", pid);
				sleep(1);
			}
	}
	return;
}


void install_signal_handlers(void) {
	char log_zeit[20];

	/* install signal handlers */
	if (signal(SIGHUP, handle_signal) == SIG_ERR)
		fprintf(stdout, "  Warning - Handler for SIGHUP was not installed for %u.\n", getpid());
	else DEBUG_FPRINTF(stdout, "  Installed handler for SIGHUP (1) in process %u.\n", getpid());
	
	if (signal(SIGINT, handle_signal) == SIG_ERR) 
		fprintf(stdout, "  Warning - Handler for SIGINT was not installed for %u.\n", getpid());
	else DEBUG_FPRINTF(stdout, "  Installed handler for SIGINT (1) in process %u.\n", getpid());

	if (signal(SIGQUIT, handle_signal) == SIG_ERR) 
		fprintf(stdout, "  Warning - Handler for SIGQUIT was not installed for %u.\n", getpid());
	else DEBUG_FPRINTF(stdout, "  Installed handler for SIGQUIT (1) in process %u.\n", getpid());
	
	if (signal(SIGTERM, handle_signal) == SIG_ERR)
		fprintf(stdout, "  Warning - Handler for SIGTERM was not installed for %u.\n", getpid());
	else DEBUG_FPRINTF(stdout, "  Installed handler for SIGTERM (1) in process %u.\n", getpid());

	if (signal(SIGCHLD, handle_signal) == SIG_ERR)
		fprintf(stdout, "  Warning - Handler for SIGCHLD was not installed for %u.\n", getpid());
	else DEBUG_FPRINTF(stdout, "  Installed handler for SIGCHLD (1) in process %u.\n", getpid());

	return;
}

