/* sniffer.c
 * Copyright (C) 2006 Tillmann Werner <tillmann.werner@gmx.de>
 *
 * This file is free software; as a special exception the author gives
 * unlimited permission to copy and/or distribute it, with or without
 * modifications, as long as this notice is preserved.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

#include <errno.h>

#include "sniffer.h"
#include "honeytrap.h"
#include "logging.h"
#include "tcpserver.h"


int start_sniffer(void) {
	char errbuf[PCAP_ERRBUF_SIZE];
	struct bpf_program filter;

	/* sniff RST packets */
	if((tcp_sniffer = pcap_open_live(dev, BUFSIZ, promisc_mode, 0, errbuf))>0) {
		/* compile bpf for tcp RST fragments */
		if (pcap_compile(tcp_sniffer, &filter, bpf_filter_string, 1, net) == -1) {
                	logmsg(LOG_ERR, 1, "Pcap error - Invalid BPF string: %s.\n", errbuf);
                	clean_exit(0);
        	}
		if (pcap_setfilter(tcp_sniffer, &filter) == -1) {
			logmsg(LOG_ERR, 1, "Pcap error - Unable to start tcp sniffer: %s\n", errbuf);
			clean_exit(0);
		}

		logmsg(LOG_NOTICE, 1, "---- Trapping attacks on %s. ----\n", dev);
		pcap_loop(tcp_sniffer, -1, start_tcp_server, NULL);

		pcap_close(tcp_sniffer);
	} else {
		logmsg(LOG_ERR, 1, "Error - %s.\n", errbuf);
		logmsg(LOG_ERR, 1, "Do you have root privileges?\n");
		clean_exit(0);
	}
	return(1);
}
